package net.wanji.opt.service.impl;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.CrossStatusEnum;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.TimeArrayUtil;
import net.wanji.databus.bo.CrossIdAndStartEndDateBO;
import net.wanji.databus.dao.entity.CrossSectionPO;
import net.wanji.databus.dao.mapper.BaseCrossSectionMapper;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.vo.RunningEvaluateCrossListVO;
import net.wanji.opt.bo.CrossNameBO;
import net.wanji.opt.service.RunningEvaluateService;
import net.wanji.opt.vo.RunningEvaluateIndexStatusVO;
import net.wanji.opt.vo.RunningEvaluateCrossEvaluateVO;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author Kent HAN
 * @date 2023/8/21 9:53
 */
@Slf4j
@Service
public class RunningEvaluateServiceImpl implements RunningEvaluateService {
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final CrossDataHistMapper crossDataHistMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;

    public RunningEvaluateServiceImpl(CrossDataRealtimeMapper crossDataRealtimeMapper,
                                      CrossDataHistMapper crossDataHistMapper,
                                      @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper) {
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.crossDataHistMapper = crossDataHistMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
    }

    @Override
    public List<RunningEvaluateCrossListVO> crossList(CrossNameBO crossNameBO) {
        String crossName = crossNameBO.getCrossName();
        List<RunningEvaluateCrossListVO> crossDataRealtimePOList = crossDataRealtimeMapper.selectByCrossName(crossName);
        for (RunningEvaluateCrossListVO runningEvaluateCrossListVO : crossDataRealtimePOList) {
            Double sturation = runningEvaluateCrossListVO.getSturation();
            runningEvaluateCrossListVO.setServiceLevel(CrossUtil.getServiceLevel(sturation));
        }
        // 按服务水平降序排序
        List<RunningEvaluateCrossListVO> res = crossDataRealtimePOList.stream()
                .sorted(Comparator.comparing(RunningEvaluateCrossListVO::getSturation).reversed())
                .collect(Collectors.toList());

        return res;
    }

    @Override
    public RunningEvaluateCrossEvaluateVO crossEvaluate(CrossIdAndStartEndDateBO bo) {
        RunningEvaluateCrossEvaluateVO vo = new RunningEvaluateCrossEvaluateVO();

        String crossId = bo.getCrossId();
        List<CrossDataHistPO> crossDataHistPOList = buildCrossDataHistPOList(bo, crossId);

        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        Double sturation = crossDataRealtimePO.getSturation();
        String serviceLevel = CrossUtil.getServiceLevel(sturation);
        vo.setServiceLevel(serviceLevel);


        List<CrossDataHistPO> congestionEventList = buildCongestionEvents(crossDataHistPOList); // 获取拥堵事件集合
        vo.setCongestionTimes(congestionEventList.size());
        List<CrossDataHistPO> unbalanceEventList = buildUnbalanceEvents(crossDataHistPOList); // 获取失衡事件集合
        vo.setUnbalanceTimes(unbalanceEventList.size());
        List<CrossDataHistPO> spilloverEventList = buildSpilloverEvents(crossDataHistPOList);  // 获取溢出事件集合
        vo.setSpilloverTimes(spilloverEventList.size());
        Integer schemeProblems = calcSchemeProblems(
                congestionEventList, unbalanceEventList, spilloverEventList, crossId); // 计算方案问题
        vo.setSchemeProblems(schemeProblems);

        return vo;
    }

    private List<CrossDataHistPO> buildCrossDataHistPOList(CrossIdAndStartEndDateBO bo, String crossId) {
        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();
        endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endStamp = (int) (endDate.getTime() / 1000);
        List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startStamp, endStamp);
        return crossDataHistPOList;
    }

    @Override
    public List<RunningEvaluateIndexStatusVO> congestionStatus(CrossIdAndStartEndDateBO bo) {

        String crossId = bo.getCrossId();
        int status = CrossStatusEnum.CONGESTION.getCode();

        return buildRes(bo, crossId, status);
    }

    @NotNull
    private List<RunningEvaluateIndexStatusVO> buildRes(CrossIdAndStartEndDateBO bo, String crossId, int status) {
        List<CrossDataHistPO> crossDataHistPOList = buildCrossDataHistPOList(bo, crossId);
        List<CrossDataHistPO> collect = crossDataHistPOList.stream()
                .filter(po -> po.getStatus() == status)
                .collect(Collectors.toList());

        List<RunningEvaluateIndexStatusVO> res = buildRunningEvaluateIndexStatusVOList(collect, status);
        return res;
    }

    @Override
    public List<RunningEvaluateIndexStatusVO> unbalanceStatus(CrossIdAndStartEndDateBO bo) {

        String crossId = bo.getCrossId();
        int status = CrossStatusEnum.UNBALANCE.getCode();

        return buildRes(bo, crossId, status);
    }

    @NotNull
    private List<RunningEvaluateIndexStatusVO> buildRunningEvaluateIndexStatusVOList(
            List<CrossDataHistPO> crossDataHistPOList, int status) {

        SimpleDateFormat sdf = new SimpleDateFormat("HH");

        Map<String, List<CrossDataHistPO>> timePOMap = crossDataHistPOList.stream()
                .collect(Collectors.groupingBy(po -> {
                    Date date = new Date((long) po.getBatchTime() * 1000); // 将10位时间戳转换为Date对象
                    return sdf.format(date); // 将Date对象转换为"HH"格式的字符串
                }));

        // 构造结果集
        List<RunningEvaluateIndexStatusVO> res = new ArrayList<>();
        List<String> timeArray = TimeArrayUtil.getTimeArray();
        for (String time : timeArray) {
            RunningEvaluateIndexStatusVO vo = new RunningEvaluateIndexStatusVO();
            vo.setTime(time);
            List<CrossDataHistPO> subList = timePOMap.get(time.substring(0, 2)); // 保留小时字符串
            if (subList != null) {
                Double avgCongestionIndex = calcAvgIndex(subList, status);
                vo.setIndex(avgCongestionIndex);
            }

            res.add(vo);
        }
        return res;
    }

    private Double calcAvgIndex(List<CrossDataHistPO> subList, int status) {

        Double average = 0.0;

        if (status == CrossStatusEnum.CONGESTION.getCode()) {
            average = subList.stream()
                    .mapToDouble(CrossDataHistPO::getCongestionIndex)
                    .average()
                    .orElse(0.0);
        } else if (status == CrossStatusEnum.UNBALANCE.getCode()) {
            average = subList.stream()
                    .mapToDouble(CrossDataHistPO::getUnbalanceIndex)
                    .average()
                    .orElse(0.0);
        }

        DecimalFormat df = new DecimalFormat("#.00");
        String formattedAverage = df.format(average);
        return Double.parseDouble(formattedAverage);
    }

    private Integer calcSchemeProblems(List<CrossDataHistPO> congestionEventList,
                                       List<CrossDataHistPO> unbalanceEventList,
                                       List<CrossDataHistPO> spilloverEventList,
                                       String crossId) {

        Set<Integer> schemeIdSet = new HashSet<>(); // 用于方案ID去重

        List<CrossDataHistPO> allList = Stream.of(congestionEventList, unbalanceEventList, spilloverEventList)
                .flatMap(List::stream)
                .collect(Collectors.toList());

        for (CrossDataHistPO abnormalEvent : allList) {
            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
            Date abnormalEventStartTime = abnormalEvent.getStartTime();
            Integer duration = abnormalEvent.getDuration();
            Date abnormalEventEndTime = DateUtil.offsetMinute(abnormalEventStartTime, duration);
            String eventStartTime = sdf.format(abnormalEventStartTime);
            String eventEndTime = sdf.format(abnormalEventEndTime);

            List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectByCrossId(crossId);
            for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                String sectionStartTime = crossSectionPO.getStartTime();
                String sectionEndTime = crossSectionPO.getEndTime();
                // 比较两组时间段是否相交
                boolean isIntersecting = isTimeIntersecting(
                        eventStartTime, eventEndTime, sectionStartTime, sectionEndTime);
                if (isIntersecting) {
                    Integer schemeId = crossSectionPO.getSchemeId();
                    schemeIdSet.add(schemeId);
                }
            }
        }

        return schemeIdSet.size();
    }

    private boolean isTimeIntersecting(String startTime1, String endTime1, String startTime2, String endTime2) {
        return !(endTime1.compareTo(startTime2) <= 0 || startTime1.compareTo(endTime2) >= 0);
    }


    private List<CrossDataHistPO> buildSpilloverEvents(List<CrossDataHistPO> crossDataHistPOList) {
        return buildEvents(crossDataHistPOList, CrossStatusEnum.SPILLOVER.getCode());
    }

    private List<CrossDataHistPO> buildUnbalanceEvents(List<CrossDataHistPO> crossDataHistPOList) {
        return buildEvents(crossDataHistPOList, CrossStatusEnum.UNBALANCE.getCode());
    }

    private List<CrossDataHistPO> buildCongestionEvents(List<CrossDataHistPO> crossDataHistPOList) {
        return buildEvents(crossDataHistPOList, CrossStatusEnum.CONGESTION.getCode());
    }

    private List<CrossDataHistPO> buildEvents(List<CrossDataHistPO> crossDataHistPOList, int status) {

        List<CrossDataHistPO> collect = crossDataHistPOList.stream()
                .filter(po -> po.getStatus() == status)
                .collect(Collectors.groupingBy(
                        CrossDataHistPO::getStartTime,
                        Collectors.collectingAndThen(
                                Collectors.maxBy(Comparator.comparingInt(CrossDataHistPO::getDuration)),
                                Optional::get
                        )
                ))
                .values()
                .stream()
                .collect(Collectors.toList());

        return collect;
    }

}
